/*
 * arch/arm/include/asm/outercache.h
 *
 * Copyright (C) 2010 ARM Ltd.
 * Written by Catalin Marinas <catalin.marinas@arm.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef __ASM_OUTERCACHE_H
#define __ASM_OUTERCACHE_H

#include <linux/types.h>
#include <mstar/mpatch_macro.h>

struct outer_cache_fns {
	#if (MP_PLATFORM_ARM == 1)
	int (*is_enable)(void); 
	#endif	/*MP_PLATFORM_ARM */
	void (*inv_range)(unsigned long, unsigned long);
	void (*clean_range)(unsigned long, unsigned long);
	#if (MP_PLATFORM_ARM == 1)
	void (*clean_all)(void);
	#endif /*MP_PLATFORM_ARM*/
	void (*flush_range)(unsigned long, unsigned long);
	void (*flush_all)(void);
	void (*inv_all)(void);
	void (*disable)(void);
#ifdef CONFIG_OUTER_CACHE_SYNC
	void (*sync)(void);
#endif
	void (*set_debug)(unsigned long);
	void (*resume)(void);
};

#ifdef CONFIG_OUTER_CACHE

extern struct outer_cache_fns outer_cache;

static inline void outer_inv_range(phys_addr_t start, phys_addr_t end)
{
	if (outer_cache.inv_range)
		outer_cache.inv_range(start, end);
}
static inline void outer_clean_range(phys_addr_t start, phys_addr_t end)
{
	if (outer_cache.clean_range)
		outer_cache.clean_range(start, end);
}
static inline void outer_flush_range(phys_addr_t start, phys_addr_t end)
{
	if (outer_cache.flush_range)
		outer_cache.flush_range(start, end);
}

static inline void outer_flush_all(void)
{
	if (outer_cache.flush_all)
		outer_cache.flush_all();
}

static inline void outer_inv_all(void)
{
	if (outer_cache.inv_all)
		outer_cache.inv_all();
}

static inline void outer_disable(void)
{
	if (outer_cache.disable)
		outer_cache.disable();
}

static inline void outer_resume(void)
{
	if (outer_cache.resume)
		outer_cache.resume();
}

#else

extern void  Chip_Flush_Miu_Pipe(void);
static inline void outer_inv_range(phys_addr_t start, phys_addr_t end)
{
    Chip_Flush_Miu_Pipe();       
}
static inline void outer_clean_range(phys_addr_t start, phys_addr_t end)
{ 
    Chip_Flush_Miu_Pipe();       

}
static inline void outer_flush_range(phys_addr_t start, phys_addr_t end)
{ 
    Chip_Flush_Miu_Pipe();       
}
static inline void outer_flush_all(void) 
{ 

    Chip_Flush_Miu_Pipe();       
}
static inline void outer_inv_all(void) 
{ 
    Chip_Flush_Miu_Pipe();       

}
static inline void outer_disable(void) 
{ 
    Chip_Flush_Miu_Pipe();       
}
static inline void outer_resume(void) 
{ 
    Chip_Flush_Miu_Pipe();       
}


#endif

#ifdef CONFIG_OUTER_CACHE_SYNC
static inline void outer_sync(void)
{
	if (outer_cache.sync)
		outer_cache.sync();
}
#else
static inline void outer_sync(void)
{ }
#endif

#endif	/* __ASM_OUTERCACHE_H */
