/*
 * Copyright (C) 2012 MStar Semiconductor Inc. All rights reserved.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#ifndef	__MS_USB_OTG_CONTROLLER__
#define	__MS_USB_OTG_CONTROLLER__

#include <linux/types.h>

/* Command Register Bit Masks */
#define USBCMD_RUN_STOP		(0x00000001)
#define USBCMD_CTRL_RESET	(0x00000002)

#define CAPLENGTH_MASK		(0xff)

/* Timer's interval, unit ms */
#define T_A_WAIT_VRISE		1000
#define T_A_WAIT_BCON		20000
#define T_A_AIDL_BDIS		1000
#define T_A_BIDL_ADIS		200
#define T_B_ASE0_BRST		4000
#define T_B_SE0_SRP		3000
#define T_B_SRP_FAIL		20000
#define T_B_DATA_PLS		100
#define T_B_SRP_INIT		1000
#define T_A_SRP_RSPNS		100
#define T_A_DRV_RSM		50

enum otg_function {
	OTG_B_DEVICE = 0,
	OTG_A_DEVICE
};

enum ms_otg_timer {
	A_WAIT_BCON_TIMER = 0,
	A_WAIT_VRISE_TIMER,
	OTG_TIMER_NUM
};

/* PXA OTG state machine */
struct ms_otg_ctrl {
	/* internal variables */
	u8 a_set_b_hnp_en; /* A-Device set b_hnp_en */
	u8 b_srp_done;
	u8 b_hnp_en;

	/* OTG inputs */
	u8 a_bus_drop;
	u8 a_bus_req;
	u8 a_clr_err;
	u8 a_bus_resume;
	u8 a_bus_suspend;
	u8 a_conn;
	u8 a_sess_vld;
	u8 a_srp_det;
	u8 a_vbus_vld;
	u8 b_bus_req; /* B-Device Require Bus */
	u8 b_bus_resume;
	u8 b_bus_suspend;
	u8 b_conn;
	u8 b_se0_srp;
	u8 b_sess_end;
	u8 b_sess_vld;
	u8 id;
	u8 a_suspend_req;

	/* Timer event */
	u8 a_aidl_bdis_timeout;
	u8 b_ase0_brst_timeout;
	u8 a_bidl_adis_timeout;
	u8 a_wait_bcon_timeout;

	struct timer_list timer[OTG_TIMER_NUM];
};

struct ms_usbc_regs {
	/* Mstar USBC register, only the LSW available */
	u32	rst_ctrl;    // 0x0
	u32	port_ctrl;   // 0x2
	u32	intr_en;     // 0x4
	u32	intr_status; // 0x6
	u32	utmi_signal; // 0x8
	u32	pwr_en;      // 0xA
	u32	pwr_status;  // 0xC
	u32	reserved;    // 0xE
};

 /* to combine USBC, UHC, and OTG */
struct ms_otg_regs {
	struct ms_usbc_regs __iomem *usbc_regs;
	void __iomem *uhc_regs;
	void __iomem *motg_regs;
	void __iomem *utmi_regs;
};

//--[USBC Reg]----------------
/* RST_CTRL */
#define  OTG_XIU         (1<<6)
#define  UHC_XIU         (1<<5)
#define  REG_SUSPEND     (1<<3)
#define  OTG_RST         (1<<2)
#define  UHC_RST         (1<<1)
#define  USB_RST         (1<<0)

/* PORT_CTRL */
#define  IDPULLUP_CTRL   (1<<4)
#define  PORTCTRL_OTG    (1<<1)
#define  PORTCTRL_UHC    (1<<0)

#define  ID_CHG_INTEN    (1<<3)
#define  ID_CHG_STS      (1<<3)
#define  IDDIG           (1<<3)

//--[EHCI Reg]----------------
#define EHC_CMD        (0x10<<1)
#define ASY_EN         (1<<5)
#define PSCH_EN        (1<<4)
#define RUN_STOP       (1<<0)
#define EHC_CMD_INT_THRC    (0x12<<1)
#define EHC_STATUS     (0x14<<1)
	
#define HC_HALTED      (1<<12)
#define EHC_INTR       (0x18<<1)
#define EHC_BMCS       (0x40<<1)
#define VBUS_OFF       (1<<4)
#define INT_POLARITY   (1<<3)

//--[OTG Reg]-----------------
#define OffShift 1

#define M_REG_FADDR        (0x00) /* 8 bit */
#define M_REG_POWER        (0x01) /* 8 bit */
#define M_REG_INTRTX       (0x02<<OffShift)
#define M_REG_INTRRX       (0x04<<OffShift)
#define M_REG_INTRTXE      (0x06<<OffShift)
#define M_REG_INTRRXE      (0x08<<OffShift)
#define M_REG_INTRUSB      (0x0A<<OffShift) /* 8 bit */
#define M_REG_INTRUSBE     ((0x0A<<OffShift)+1) /* 8 bit */

#define	M_POWER_SOFTCONN    0x4000 // at address 1

struct ms_otg {
	struct usb_phy phy;
	struct ms_otg_ctrl otg_ctrl;

	/* base address */
	void __iomem *phy_regs;
	void __iomem *cap_regs;
	struct ms_otg_regs __iomem *op_regs;

	struct platform_device *pdev;
	int irq; // usb int
	int irq_uhc; // uhc int
	u32 irq_status;
	u32 irq_en;

	/* state backup */
	u32 uhc_intr_status;
	u32 uhc_usbcmd_status;
	u32 uhc_usbcmd_int_thrc_status;
	u32 motg_IntrTx;
	u32 motg_IntrRx;
	u32 motg_IntrUSB;
	
	struct delayed_work work;
	struct workqueue_struct *qwork;

	spinlock_t wq_lock;

	struct ms_usb_platform_data *pdata;

	unsigned int active;
	unsigned int clock_gating;
	struct clk *clk;
};
#endif
